@@ -487,7 +487,7 @@ QString ShibokenGenerator::cpythonWrapperCPtr(const AbstractMetaType* metaType,
 QString ShibokenGenerator::cpythonWrapperCPtr(const TypeEntry* type, QString argName)
 {
     if (type->isValue() || type->isObject())
-        return baseConversionString(type->qualifiedCppName() + '*') + "toCpp(" + argName + ')';
+        return baseConversionString( "::" + type->qualifiedCppName() + '*') + "toCpp(" + argName + ')';
     return QString();
 }
 
@@ -519,8 +519,6 @@ void ShibokenGenerator::writeBaseConversion(QTextStream& s, const TypeEntry* typ
             typeName = protectedEnumSurrogateName(metaEnum);
     } else {
         typeName = type->qualifiedCppName().trimmed();
-        if (!type->isCppPrimitive())
-            typeName.prepend("::");
         if (type->isObject())
             typeName.append('*');
     }
@@ -542,8 +540,6 @@ void ShibokenGenerator::writeBaseConversion(QTextStream& s, const AbstractMetaTy
         if (type->isContainer() || type->isFlags() || type->isEnum() || (type->isConstant() && type->isReference()))
             options |= Generator::ExcludeReference;
         typeName = translateTypeForWrapperMethod(type, context, options).trimmed();
-        if (!type->typeEntry()->isCppPrimitive())
-            typeName.prepend("::");
     }
 
     s << baseConversionString(typeName);
@@ -865,6 +861,8 @@ QString ShibokenGenerator::guessCPythonCheckFunction(const QString& type)
         metaType = 0;
     } else if (type == "PyTypeObject") {
         retval = "PyType_Check";
+    } else if (type == "PyBuffer") {
+        retval = "Shiboken::Buffer::checkType";
     } else {
         retval = QString("%1_Check").arg(type);
     }
@@ -1104,6 +1102,34 @@ ShibokenGenerator::ExtendedConverterData ShibokenGenerator::getExtendedConverter
     return extConvs;
 }
 
+static QString getArgumentsFromMethodCall(const QString& str)
+{
+    // It would be way nicer to be able to use a Perl like
+    // regular expression that accepts temporary variables
+    // to count the parenthesis.
+    // For more information check this:
+    // http://perl.plover.com/yak/regex/samples/slide083.html
+    static QString funcCall("%CPPSELF.%FUNCTION_NAME");
+    int pos = str.indexOf(funcCall);
+    if (pos == -1)
+        return QString();
+    pos = pos + funcCall.count();
+    while (str.at(pos) == ' ' || str.at(pos) == '\t')
+        ++pos;
+    if (str.at(pos) == '(')
+        ++pos;
+    int begin = pos;
+    int counter = 1;
+    while (counter != 0) {
+        if (str.at(pos) == '(')
+            ++counter;
+        else if (str.at(pos) == ')')
+            --counter;
+        ++pos;
+    }
+    return str.mid(begin, pos-begin-1);
+}
+
 void ShibokenGenerator::writeCodeSnips(QTextStream& s,
                                        const CodeSnipList& codeSnips,
                                        CodeSnip::Position position,
@@ -1114,6 +1140,8 @@ void ShibokenGenerator::writeCodeSnips(QTextStream& s,
 {
     static QRegExp toPythonRegex("%CONVERTTOPYTHON\\[([^\\[]*)\\]");
     static QRegExp toCppRegex("%CONVERTTOCPP\\[([^\\[]*)\\]");
+    static QRegExp isConvertibleRegex("%ISCONVERTIBLE\\[([^\\[]*)\\]");
+    static QRegExp checkTypeRegex("%CHECKTYPE\\[([^\\[]*)\\]");
     static QRegExp pyArgsRegex("%PYARG_(\\d+)");
 
     // detect is we should use pyargs instead of args as variable name for python arguments
@@ -1144,12 +1172,18 @@ void ShibokenGenerator::writeCodeSnips(QTextStream& s,
             code.replace("%CPPTYPE", context->name());
         }
 
-        // replace "toPython "converters
+        // replace "toPython" converters
         code.replace(toPythonRegex, "Shiboken::Converter<\\1 >::toPython");
 
-        // replace "toCpp "converters
+        // replace "toCpp" converters
         code.replace(toCppRegex, "Shiboken::Converter<\\1 >::toCpp");
 
+        // replace "isConvertible" check
+        code.replace(isConvertibleRegex, "Shiboken::Converter<\\1 >::isConvertible");
+
+        // replace "checkType" check
+        code.replace(checkTypeRegex, "Shiboken::Converter<\\1 >::checkType");
+
         if (func) {
             // replace %PYARG_# variables
             code.replace("%PYARG_0", PYTHON_RETURN_VAR);
@@ -1222,6 +1256,25 @@ void ShibokenGenerator::writeCodeSnips(QTextStream& s,
                 if (func->isComparisonOperator())
                     replacement = "%1.";
 
+                if (func->isVirtual() && !func->isAbstract() && (!avoidProtectedHack() || !func->isProtected())) {
+                    QString methodCallArgs = getArgumentsFromMethodCall(code);
+                    if (!methodCallArgs.isNull()) {
+                        if (func->name() == "metaObject") {
+                            QString wrapperClassName = wrapperName(func->ownerClass());
+                            QString cppSelfVar = avoidProtectedHack() ? QString("%CPPSELF") : QString("reinterpret_cast<%1*>(%CPPSELF)").arg(wrapperClassName);
+                            code.replace(QString("%CPPSELF.%FUNCTION_NAME(%1)").arg(methodCallArgs),
+                                         QString("(Shiboken::Object::hasCppWrapper(reinterpret_cast<SbkObject*>(%1))"
+                                                 " ? %2->::%3::%FUNCTION_NAME(%4)"
+                                                 " : %CPPSELF.%FUNCTION_NAME(%4))").arg(pySelf).arg(cppSelfVar).arg(wrapperClassName).arg(methodCallArgs));
+                        } else {
+                            code.replace(QString("%CPPSELF.%FUNCTION_NAME(%1)").arg(methodCallArgs),
+                                         QString("(Shiboken::Object::hasCppWrapper(reinterpret_cast<SbkObject*>(%1))"
+                                                 " ? %CPPSELF->::%TYPE::%FUNCTION_NAME(%2)"
+                                                 " : %CPPSELF.%FUNCTION_NAME(%2))").arg(pySelf).arg(methodCallArgs));
+                        }
+                    }
+                }
+
                 code.replace("%CPPSELF.", replacement.arg(cppSelf));
                 code.replace("%CPPSELF", cppSelf);
 
