@@ -57,7 +57,7 @@ PyTypeObject SbkObjectType_Type = {
     /*tp_call*/             0,
     /*tp_str*/              0,
     /*tp_getattro*/         0,
-    /*tp_setattro*/         0,
+    /*tp_setattro*/         PyObject_GenericSetAttr,
     /*tp_as_buffer*/        0,
     /*tp_flags*/            Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,
     /*tp_doc*/              0,
@@ -76,9 +76,9 @@ PyTypeObject SbkObjectType_Type = {
     /*tp_descr_set*/        0,
     /*tp_dictoffset*/       0,
     /*tp_init*/             0,
-    /*tp_alloc*/            0,
+    /*tp_alloc*/            PyType_GenericAlloc,
     /*tp_new*/              SbkObjectTypeTpNew,
-    /*tp_free*/             0,
+    /*tp_free*/             PyObject_GC_Del,
     /*tp_is_gc*/            0,
     /*tp_bases*/            0,
     /*tp_mro*/              0,
@@ -102,6 +102,22 @@ static PyGetSetDef SbkObjectGetSetList[] = {
     {0} // Sentinel
 };
 
+static int SbkObject_traverse(PyObject* self, visitproc visit, void* arg)
+{
+    SbkObject* sbkSelf = reinterpret_cast<SbkObject*>(self);
+    if (sbkSelf->ob_dict)
+        Py_VISIT(sbkSelf->ob_dict);
+    return 0;
+}
+
+static int SbkObject_clear(PyObject* self)
+{
+    SbkObject* sbkSelf = reinterpret_cast<SbkObject*>(self);
+    if (sbkSelf->ob_dict)
+        Py_CLEAR(sbkSelf->ob_dict);
+    return 0;
+}
+
 SbkObjectType SbkObject_Type = { { {
     PyObject_HEAD_INIT(&SbkObjectType_Type)
     /*ob_size*/             0,
@@ -123,10 +139,10 @@ SbkObjectType SbkObject_Type = { { {
     /*tp_getattro*/         0,
     /*tp_setattro*/         0,
     /*tp_as_buffer*/        0,
-    /*tp_flags*/            Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,
+    /*tp_flags*/            Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
     /*tp_doc*/              0,
-    /*tp_traverse*/         0,
-    /*tp_clear*/            0,
+    /*tp_traverse*/         SbkObject_traverse,
+    /*tp_clear*/            SbkObject_clear,
     /*tp_richcompare*/      0,
     /*tp_weaklistoffset*/   offsetof(SbkObject, weakreflist),
     /*tp_iter*/             0,
@@ -172,7 +188,8 @@ void SbkDeallocWrapper(PyObject* pyObj)
             sbkType->d->cpp_dtor(sbkObj->d->cptr[0]);
         }
     }
-    Shiboken::Object::deallocData(sbkObj, !sbkObj->d->containsCppWrapper);
+    //Always destroy object data during the python object destruction
+    Shiboken::Object::deallocData(sbkObj, true);
 }
 
 void SbkDeallocWrapperWithPrivateDtor(PyObject* self)
@@ -187,9 +204,9 @@ void SbkDeallocWrapperWithPrivateDtor(PyObject* self)
 void SbkObjectTypeDealloc(PyObject* pyObj)
 {
     SbkObjectType* sbkType = reinterpret_cast<SbkObjectType*>(pyObj);
-    if (!sbkType->d)
-        return;
 
+    PyObject_GC_UnTrack(pyObj);
+    Py_TRASHCAN_SAFE_BEGIN(pyObj);
     if(sbkType->d->user_data && sbkType->d->d_func) {
         sbkType->d->d_func(sbkType->d->user_data);
         sbkType->d->user_data = 0;
@@ -198,6 +215,7 @@ void SbkObjectTypeDealloc(PyObject* pyObj)
     sbkType->d->original_name = 0;
     delete sbkType->d;
     sbkType->d = 0;
+    Py_TRASHCAN_SAFE_END(pyObj);
 }
 
 PyObject* SbkObjectTypeTpNew(PyTypeObject* metatype, PyObject* args, PyObject* kwds)
@@ -274,7 +292,8 @@ PyObject* SbkObjectTypeTpNew(PyTypeObject* metatype, PyObject* args, PyObject* k
 
 PyObject* SbkObjectTpNew(PyTypeObject* subtype, PyObject*, PyObject*)
 {
-    SbkObject* self = reinterpret_cast<SbkObject*>(subtype->tp_alloc(subtype, 0));
+    SbkObject* self = PyObject_GC_New(SbkObject, subtype);
+    Py_INCREF(reinterpret_cast<PyObject*>(subtype));
     SbkObjectPrivate* d = new SbkObjectPrivate;
 
     SbkObjectType* sbkType = reinterpret_cast<SbkObjectType*>(subtype);
@@ -290,6 +309,7 @@ PyObject* SbkObjectTpNew(PyTypeObject* subtype, PyObject*, PyObject*)
     self->ob_dict = 0;
     self->weakreflist = 0;
     self->d = d;
+    PyObject_GC_Track(reinterpret_cast<PyObject*>(self));
     return reinterpret_cast<PyObject*>(self);
 }
 
@@ -299,8 +319,7 @@ PyObject* SbkObjectTpNew(PyTypeObject* subtype, PyObject*, PyObject*)
 namespace Shiboken
 {
 
-static void incRefPyObject(PyObject* pyObj);
-static void decRefPyObjectList(const std::list<SbkObject*> &pyObj);
+static void decRefPyObjectList(const std::list<PyObject*> &pyObj, PyObject* skip = 0);
 
 void walkThroughClassHierarchy(PyTypeObject* currentType, HierarchyVisitor* visitor)
 {
@@ -455,22 +474,12 @@ std::list<SbkObject*> splitPyObject(PyObject* pyObj)
     return result;
 }
 
-static void incRefPyObject(PyObject* pyObj)
+static void decRefPyObjectList(const std::list<PyObject*>& lst, PyObject *skip)
 {
-    if (PySequence_Check(pyObj)) {
-        for(int i = 0, i_max = PySequence_Size(pyObj); i < i_max; i++) {
-            PySequence_GetItem(pyObj, i);
-        }
-    } else {
-        Py_INCREF(pyObj);
-    }
-}
-
-static void decRefPyObjectList(const std::list<SbkObject*>& lst)
-{
-    std::list<SbkObject*>::const_iterator iter = lst.begin();
+    std::list<PyObject*>::const_iterator iter = lst.begin();
     while(iter != lst.end()) {
-        Py_DECREF(*iter);
+        if (*iter != skip)
+            Py_DECREF(*iter);
         ++iter;
     }
 }
@@ -847,6 +856,15 @@ bool isValid(SbkObject* pyObj, bool throwPyError)
     return true;
 }
 
+bool isValid(PyObject* pyObj, bool throwPyError)
+{
+    if (!pyObj || pyObj == Py_None ||
+        !PyType_IsSubtype(pyObj->ob_type, reinterpret_cast<PyTypeObject*>(&SbkObject_Type))) {
+        return true;
+    }
+    return isValid(reinterpret_cast<SbkObject*>(pyObj), throwPyError);
+}
+
 PyObject* newObject(SbkObjectType* instanceType,
                     void* cptr,
                     bool hasOwnership,
@@ -923,7 +941,6 @@ void removeParent(SbkObject* child, bool giveOwnershipBack, bool keepReference)
     if (!pInfo || !pInfo->parent) {
         return;
     }
-
     ChildrenList& oldBrothers = pInfo->parent->d->parentInfo->children;
     // Verify if this child is part of parent list
     ChildrenList::iterator iChild = std::find(oldBrothers.begin(), oldBrothers.end(), child);
@@ -1059,21 +1076,48 @@ void keepReference(SbkObject* self, const char* key, PyObject* referredObject, b
         self->d->referredObjects = new Shiboken::RefCountMap;
 
     RefCountMap& refCountMap = *(self->d->referredObjects);
-    if (!isNone)
-        incRefPyObject(referredObject);
-
     RefCountMap::iterator iter = refCountMap.find(key);
-    if (!append && (iter != refCountMap.end())) {
-        decRefPyObjectList(iter->second);
-        refCountMap.erase(iter);
+    std::list<PyObject*> objects;
+    if (iter != refCountMap.end()) {
+        objects = (*iter).second;
+        std::list<PyObject*>::const_iterator found = std::find(objects.begin(), objects.end(), referredObject);
+
+        // skip if objects already exists
+        if (found != objects.end())
+            return;
     }
 
-    if (!isNone) {
-        std::list<SbkObject*> values = splitPyObject(referredObject);
-        if (append && (iter != refCountMap.end()))
-            refCountMap[key].insert(refCountMap[key].end(), values.begin(), values.end());
-        else
-            refCountMap[key] = values;
+    if (append && !isNone) {
+        refCountMap[key].push_back(referredObject);
+        Py_INCREF(referredObject);
+    } else if (!append) {
+        if (objects.size() > 0)
+            decRefPyObjectList(objects, isNone ? 0 : referredObject);
+        if (isNone) {
+            if (iter != refCountMap.end())
+                refCountMap.erase(iter);
+        } else {
+            objects.clear();
+            objects.push_back(referredObject);
+            refCountMap[key] = objects;
+            Py_INCREF(referredObject);
+        }
+    }
+}
+
+void removeReference(SbkObject* self, const char* key, PyObject* referredObject)
+{
+    if (!referredObject || (referredObject == Py_None))
+        return;
+
+    if (!self->d->referredObjects)
+        return;
+
+    RefCountMap& refCountMap = *(self->d->referredObjects);
+    RefCountMap::iterator iter = refCountMap.find(key);
+    if (iter != refCountMap.end()) {
+        decRefPyObjectList(iter->second);
+        refCountMap.erase(iter);
     }
 }
 
@@ -1090,7 +1134,7 @@ void clearReferences(SbkObject* self)
     self->d->referredObjects = 0;
 }
 
-} // namespace Wrapper
+} // namespace Object
 
 } // namespace Shiboken
 
